@@ -3,8 +3,10 @@ use strict;
 package smbldap_tools;
 use Net::LDAP;
 use Crypt::SmbHash;
+use Unicode::MapUTF8 qw(to_utf8 from_utf8);
 
-# $Id: smbldap_tools.pm,v 1.50 2004/10/28 10:13:22 jtournier Exp $
+
+# $Id: smbldap_tools.pm,v 1.65 2006/01/02 17:01:19 jtournier Exp $
 #
 #  This code was developped by IDEALX (http://IDEALX.org/) and
 #  contributors (their names can be found in the CONTRIBUTORS file).
@@ -29,6 +31,26 @@ use Crypt::SmbHash;
 
 # ugly funcs using global variables and spawning openldap clients
 
+my $smbldap_conf;
+if (-e "/etc/smbldap-tools/smbldap.conf") {
+	$smbldap_conf="/etc/smbldap-tools/smbldap.conf";
+} else {
+	$smbldap_conf="/etc/opt/IDEALX/smbldap-tools/smbldap.conf";
+}
+
+my $smbldap_bind_conf;
+if (-e "/etc/smbldap-tools/smbldap_bind.conf") {
+	$smbldap_bind_conf="/etc/smbldap-tools/smbldap_bind.conf";
+} else {
+	$smbldap_bind_conf="/etc/opt/IDEALX/smbldap-tools/smbldap_bind.conf";
+}
+my $samba_conf;
+if (-e "/etc/samba/smb.conf") {
+	$samba_conf="/etc/samba/smb.conf";
+} else {
+	$samba_conf="/usr/local/samba/lib/smb.conf";
+}
+
 use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use Exporter;
 $VERSION = 1.00;
@@ -37,53 +59,60 @@ $VERSION = 1.00;
 use vars qw(%config $ldap);
 
 @EXPORT = qw(
-			 get_user_dn
-			 get_group_dn
-			 is_group_member
-			 is_samba_user
-			 is_unix_user
-			 is_user_valid
-			 does_sid_exist
-			 get_dn_from_line
-			 add_posix_machine
-			 add_samba_machine
-			 add_samba_machine_smbpasswd
-			 group_add_user
-			 add_grouplist_user
-			 disable_user
-			 delete_user
-			 group_add
-			 group_del
-			 get_homedir
-			 read_user
-			 read_user_entry
-			 read_group
-			 read_group_entry
-			 read_group_entry_gid
-			 find_groups_of
-			 parse_group
-			 group_remove_member
-			 group_get_members
-			 do_ldapadd
-			 do_ldapmodify
-			 get_user_dn2
-			 connect_ldap_master
-			 connect_ldap_slave
-			 group_type_by_name
-			 read_config
-			 read_parameter
-			 subst_user
-			 split_arg_comma
-			 list_union
-			 list_minus
-			 get_next_id
-			 print_banner
-			 %config
-			);
+	     get_user_dn
+	     get_group_dn
+	     is_group_member
+	     is_samba_user
+	     is_unix_user
+	     is_nonldap_unix_user
+	     is_user_valid
+	     does_sid_exist
+	     get_dn_from_line
+	     add_posix_machine
+	     add_samba_machine
+	     add_samba_machine_smbpasswd
+	     group_add_user
+	     add_grouplist_user
+	     disable_user
+	     delete_user
+	     group_add
+	     group_del
+	     get_homedir
+	     read_user
+	     read_user_entry
+	     read_group
+	     read_group_entry
+	     read_group_entry_gid
+	     find_groups_of
+	     parse_group
+	     group_remove_member
+	     group_get_members
+	     do_ldapadd
+	     do_ldapmodify
+	     get_user_dn2
+	     connect_ldap_master
+	     connect_ldap_slave
+	     group_type_by_name
+	     subst_configvar
+	     read_config
+	     read_parameter
+	     subst_user
+	     split_arg_comma
+	     list_union
+	     list_minus
+	     get_next_id
+	     print_banner
+	     getDomainName
+	     getLocalSID
+	     utf8Encode
+	     utf8Decode
+	     %config
+	    );
 
 sub print_banner
   {
-	print STDERR "(c) IDEALX 2004 (http://www.idealx.com)- Licenced under GPL\n";
+       print STDERR "(c) Jerome Tournier - IDEALX 2004 (http://www.idealx.com)- Licensed under the GPL\n"
+         unless $config{no_banner};
   }
 
 sub read_parameter
@@ -92,10 +121,9 @@ sub read_parameter
 	## check for a param = value
 	if ($_=~/=/) {
 	  my ($param,$val);
-	  if ($_=~/"/) {
-		#my ($param,$val) = ($_=~/(.*)\s*=\s*"(.*)"/);
+	  if ($_=~/\s*.*?\s*=\s*".*"/) {
 		($param,$val) = /\s*(.*?)\s*=\s*"(.*)"/;
-	  } elsif ($_=~/'/) {
+	  } elsif ($_=~/\s*.*?\s*=\s*'.*'/) {
 		($param,$val) = /\s*(.*?)\s*=\s*'(.*)'/;
 	  } else {
 		($param,$val) = /\s*(.*?)\s*=\s*(.*)/;
@@ -116,7 +144,6 @@ sub subst_configvar
 sub read_conf
   {
 	my %conf;
-	my $smbldap_conf="/etc/smbldap-tools/smbldap.conf";
 	open (CONFIGFILE, "$smbldap_conf") || die "Unable to open $smbldap_conf for reading !\n";
 	while (<CONFIGFILE>) {
 	  chomp($_);
@@ -130,8 +157,7 @@ sub read_conf
 	close (CONFIGFILE);
 
 	if ($< == 0) {
-	  $smbldap_conf="/etc/smbldap-tools/smbldap_bind.conf";
-	  open (CONFIGFILE, "$smbldap_conf") || die "Unable to open $smbldap_conf for reading !\n";
+	  open (CONFIGFILE, "$smbldap_bind_conf") || die "Unable to open $smbldap_bind_conf for reading !\n";
 	  while (<CONFIGFILE>) {
 		chomp($_);
 		## throw away comments
@@ -145,13 +171,110 @@ sub read_conf
 	} else {
 	  $conf{slaveDN}=$conf{slavePw}=$conf{masterDN}=$conf{masterPw}="";
 	}
-	
+       # automatically find SID
+       if (not $conf{SID}) {
+         $conf{SID} = getLocalSID() ||
+           die "Unable to determine domain SID: please edit your smbldap.conf,
+  or start your samba server for a few minutes to allow for SID generation to proceed\n";
+       }
 	return(%conf);
   }
 
+sub read_smbconf
+  {
+    my %conf;
+    my $smbconf="$samba_conf";
+    open (CONFIGFILE, "$smbconf") || die "Unable to open $smbconf for reading !\n";
+    my $global=0;
+    my $prevline="";
+    while (<CONFIGFILE>) {
+     chomp;
+     if (/^(.*)\\$/) {
+	$prevline.=$1;
+	next;
+     }
+     $_=$prevline.$_;
+     $prevline="";
+      if (/^\[global\]/) {
+	$global=1;
+      }
+      if ($global == 1) {
+	if (/^\[/ and !/\[global\]/) {
+	  $global=0;
+	} else {
+  	  ## throw away comments
+  	  #next if ( ! /workgroup/i );
+	  next if ( /^\s*#/ || /^\s*$/ || /^\s*\;/ || /\[/);
+	  ## check for a param = value
+	  my ($parameter,$value)=read_parameter($_);
+	  $value = &subst_configvar($value, \%conf);
+	  $conf{$parameter}=$value;
+	}
+      }
+    }
+    close (CONFIGFILE);
+    return(%conf);
+  }
+my %smbconf=read_smbconf();
+
+sub getLocalSID {
+  my $string = `LANG= PATH=/opt/IDEALX/bin:/usr/local/bin:/usr/bin:/bin net getlocalsid 2>/dev/null`;
+  my ($domain,$sid)=($string =~ m/^SID for domain (\S+) is: (\S+)$/ );
+
+  return $sid;
+}
+
 # let's read the configurations file...
 %config=read_conf();
 
+sub get_parameter {
+  # this function return the value for a parameter. The name of the parameter can be either this
+  # defined in smb.conf or smbldap.conf
+  my $parameter_smb=shift;
+  my $parameter_smbldap=shift;
+  if (defined $config{$parameter_smbldap} and $config{$parameter_smbldap} ne "") {
+	return $config{$parameter_smbldap};
+  } elsif (defined $smbconf{$parameter_smb} and $smbconf{$parameter_smb} ne "") {
+        return $smbconf{$parameter_smb};
+  } else {
+	#print "could not find parameter's value (parameter given: $parameter_smbldap or $parameter_smb) !!\n";
+	undef $smbconf{$parameter_smb};
+  }
+  
+}
+
+$config{sambaDomain}=get_parameter("workgroup","sambaDomain");
+$config{suffix}=get_parameter("ldap suffix","suffix");
+$config{usersdn}=get_parameter("ldap user suffix","usersdn");
+if ($config{usersdn} !~ m/,/ ) {$config{usersdn}=$config{usersdn}.",".$config{suffix};}
+$config{groupsdn}=get_parameter("ldap group suffix","groupsdn");
+if ($config{groupsdn} !~ m/,/ ) {$config{groupsdn}=$config{groupsdn}.",".$config{suffix};}
+$config{computersdn}=get_parameter("ldap machine suffix","computersdn");
+if ($config{computersdn} !~ m/,/ ) {$config{computersdn}=$config{computersdn}.",".$config{suffix};}
+$config{idmapdn}=get_parameter("ldap idmap suffix","idmapdn");
+if (defined $config{idmapdn}) {
+	if ($config{idmapdn} !~ m/,/ ) {$config{idmapdn}=$config{idmapdn}.",".$config{suffix};}
+}
+
+# next uidNumber and gidNumber available are stored in sambaDomainName object
+if (!defined $config{sambaUnixIdPooldn}) {
+	$config{sambaUnixIdPooldn}="sambaDomainName=$config{sambaDomain},$config{suffix}";
+}
+if (!defined $config{masterLDAP}) {
+	$config{masterLDAP}="127.0.0.1";
+}
+if (!defined $config{masterPort}) {
+	$config{masterPort}="389";
+}
+if (!defined $config{slaveLDAP}) {
+	$config{slaveLDAP}="127.0.0.1";
+}
+if (!defined $config{slavePort}) {
+	$config{slavePort}="389";
+}
+if (!defined $config{ldapTLS}) {
+	$config{ldapTLS}="0";
+}
 
 sub connect_ldap_master
   {
@@ -312,6 +435,19 @@ sub is_unix_user
 	return ($mesg->count ne 0);
   }
 
+sub is_nonldap_unix_user
+  {
+	my $user = shift;
+	my $uid = getpwnam($user);
+
+	if ($uid) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
+
 sub is_group_member
   {
 	my $dn_group = shift;
@@ -342,21 +478,47 @@ sub does_sid_exist
 sub is_user_valid
   {
 	my ($user, $dn, $pass) = @_;
-	my $userLdap = Net::LDAP->new($config{slaveLDAP}) or die "erreur LDAP";
-	my $mesg= $userLdap->bind (dn => $dn, password => $pass );
-	if ($mesg->code eq 0) {
-	  $userLdap->unbind;
-	  return 1;
-	} else {
-	  if ($userLdap->bind()) {
-		$userLdap->unbind;
-		return 0;
+       my $userLdap = Net::LDAP->new(
+                                                                "$config{slaveLDAP}",
+                                                                port => "$config{slavePort}",
+                                                                version => 3,
+                                                                timeout => 60
+                                                                )
+         or warn "erreur LDAP: Can't contact slave ldap server ($@)\n=>trying to contact the master server\n";
+       if (!$userLdap) {
+         # connection to the slave failed: trying to contact the master ...
+         $userLdap = Net::LDAP->new(
+                                                                "$config{masterLDAP}",
+                                                                port => "$config{masterPort}",
+                                                                version => 3,
+                                                                timeout => 60
+                                                                )
+               or die "erreur LDAP: Can't contact master ldap server ($@)\n";
+       }
+       if ($userLdap) {
+         if ($config{ldapTLS} == 1) {
+               $userLdap->start_tls(
+                                                          verify => "$config{verify}",
+                                                          clientcert => "$config{clientcert}",
+                                                          clientkey => "$config{clientkey}",
+                                                          cafile => "$config{cafile}"
+                                                         );
+         }
+         my $mesg= $userLdap->bind (dn => $dn, password => $pass );
+         if ($mesg->code eq 0) {
+           $userLdap->unbind;
+           return 1;
 	  } else {
-		print ("The LDAP directory is not available.\n Check the server, cables ...");
-		$userLdap->unbind;
-		return 0;
+           if ($userLdap->bind()) {
+             $userLdap->unbind;
+             return 0;
+           } else {
+             print ("The LDAP directory is not available.\n Check the server, cables ...");
+             $userLdap->unbind;
+             return 0;
+           }
+           die "Problem : contact your administrator";
 	  }
-	  die "Problem : contact your administrator";
 	}
   }
 
@@ -374,11 +536,14 @@ sub get_dn_from_line
 # success = add_posix_machine($user, $uid, $gid)
 sub add_posix_machine
   {
-	my ($user, $uid, $gid) = @_;
+	my ($user,$uid,$gid,$wait) = @_;
+	if (!defined $wait) {
+		$wait=0;
+	}
 	# bind to a directory with dn and password
 	my $add = $ldap->add ( "uid=$user,$config{computersdn}",
 								  attr => [
-										   'objectclass' => ['top','inetOrgPerson', 'posixAccount'],
+										   'objectclass' => ['top', 'person', 'organizationalPerson', 'inetOrgPerson', 'posixAccount'],
 										   'cn'   => "$user",
 										   'sn'   => "$user",
 										   'uid'   => "$user",
@@ -392,7 +557,7 @@ sub add_posix_machine
 								);
 	
 	$add->code && warn "failed to add entry: ", $add->error ;
-	# take down the session
+	sleep($wait);
 	return 1;
   }
 
@@ -550,7 +715,7 @@ sub group_add
 	}
 	my $modify = $ldap->add ( "cn=$gname,$config{groupsdn}",
 									 attrs => [
-											   objectClass => 'posixGroup',
+											   objectClass => [ 'top', 'posixGroup' ],
 											   cn => "$gname",
 											   gidNumber => "$gid"
 											  ]
@@ -905,9 +1070,23 @@ sub get_next_id($$) {
   die "Could not allocate $attribute!";
 }
 
+sub utf8Encode {
+  my $arg = shift; 
 
-#$ldap->unbind;
-#$ldap->unbind;
+  return to_utf8(
+                                 -string=> $arg,
+                                 -charset => 'ISO-8859-1',
+                                );
+}
+
+sub utf8Decode {
+  my $arg = shift;
+
+  return from_utf8(
+                                   -string=> $arg,
+                                   -charset => 'ISO-8859-1',
+                                  );
+}   
 
 1;
 
